@@ -10,14 +10,14 @@
 # Python 2 and 3 compatibility utilities
 import six
 
-from bitcoin.mixins import HashableMixin, SerializableMixin
-from bitcoin.serialize import (
+from .mixins import HashableMixin, SerializableMixin
+from .serialize import (
     serialize_varint, deserialize_varint,
     serialize_varchar, deserialize_varchar,
     serialize_hash, deserialize_hash,
     serialize_list, deserialize_list)
 
-from .tools import Bits, StringIO, icmp, list, tuple
+from .tools import Bits, StringIO, icmp, lookahead, list, tuple
 
 __all__ = (
     'PatriciaLink',
@@ -32,9 +32,9 @@ SENTINAL = object()
 import numbers
 
 class PatriciaLink(HashableMixin):
-    __slots__ = 'prefix node _hash'.split()
+    __slots__ = 'prefix node _hash _size'.split()
 
-    def __init__(self, prefix, node=None, hash=None, *args, **kwargs):
+    def __init__(self, prefix, node=None, hash=None, size=0, *args, **kwargs):
         if not isinstance(prefix, Bits):
             if isinstance(prefix, six.binary_type):
                 prefix = Bits(bytes=prefix)
@@ -42,13 +42,27 @@ class PatriciaLink(HashableMixin):
                 prefix = Bits(prefix)
         if isinstance(node, numbers.Integral):
             node, hash = hash, node
+        if hasattr(node, 'size'):
+            size = node.size
         super(PatriciaLink, self).__init__(*args, **kwargs)
-        self.prefix, self.node, self._hash = prefix, node, hash
+        self.prefix, self.node, self._hash, self._size = prefix, node, hash, size
 
     @property
     def pruned(self):
         return self.node is None
 
+    @property
+    def size(self):
+        if getattr(self, '_size', None) is None:
+            self._size = getattr(self.node, 'size', 0)
+        return self._size
+
+    @property
+    def length(self):
+        if self.pruned:
+            return 0
+        return self.node.length
+
     def hash__getter(self):
         if getattr(self, '_hash', None) is not None:
             return self._hash
@@ -80,7 +94,7 @@ class PatriciaLink(HashableMixin):
     def __repr__(self):
         parts = []
         parts.append('prefix=%s' % repr(self.prefix))
-        parts.append('hash=0x%064x' % (self.hash or 0))
+        parts.append((self.pruned and 'hash' or 'node') + '=0x%064x' % (self.hash or 0))
         return '%s(%s)' % (self.__class__.__name__, ', '.join(parts))
 
 # ===----------------------------------------------------------------------===
@@ -92,7 +106,7 @@ from struct import pack, unpack
 class BasePatriciaDict(SerializableMixin, HashableMixin):
     """An ordered dictionary implemented with a hybrid
     level- and node-compressed prefix tree."""
-    __slots__ = 'value children _hash size length'.split()
+    __slots__ = 'value prune_value children _hash size length'.split()
 
     OFFSET_LEFT  = 0
     OFFSET_RIGHT = 2
@@ -184,9 +198,8 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
         size = int(value is not None)
         length = size - int(prune_value is True)
         for link in children:
-            if not link.pruned:
-                size += link.node.size
-                length += link.node.length
+            size += link.size
+            length += link.length
         self.size, self.length = size, length
 
     @property
@@ -342,27 +355,27 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
         path = [(self, 0, Bits())]
         while path:
             node, idx, prefix = path.pop()
-            if idx==0 and node.value is not None:
+            if idx==0 and node.value is not None and not node.prune_value:
                 yield (self._unpickle_key(prefix), self._unpickle_value(node.value))
             if idx<len(node.children):
                 path.append((node, idx+1, prefix))
-                path.append((
-                    node.children[idx].node,
-                    0,
-                    prefix + node.children[idx].prefix))
+                link = node.children[idx]
+                if not link.pruned:
+                    path.append((link.node, 0, prefix + link.prefix))
 
     def _reverse_iterator(self):
         "Returns a reverse/backwards iterator over the trie"
         path = [(self, len(self.children)-1, Bits())]
         while path:
             node, idx, prefix = path.pop()
-            if idx<0 and node.value is not None:
+            if idx<0 and node.value is not None and not node.prune_value:
                 yield (self._unpickle_key(prefix), self._unpickle_value(node.value))
             if idx>=0:
                 path.append((node, idx-1, prefix))
                 link = node.children[idx]
-                node = link.node
-                path.append((node, len(node.children)-1, prefix + link.prefix))
+                if not link.pruned:
+                    node = link.node
+                    path.append((node, len(node.children)-1, prefix + link.prefix))
 
     def items(self):
         "x.items() -> list of x's (key, value) pairs, as 2-tuples in sorted order"
@@ -421,6 +434,8 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
         while prefix != key:
             for idx,link in enumerate(node.children):
                 if subkey.startswith(link.prefix):
+                    if link.pruned:
+                        return (prefix, node)
                     subkey = subkey[len(link.prefix):]
                     prefix += link.prefix
                     if path is not None:
@@ -464,11 +479,15 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
             lambda: getattr(self, 'node_class', self.__class__))()
         while path:
             parent, idx, prefix = path.pop()
+            if node.length:
+                link = link_class(prefix=prefix, node=node)
+            else:
+                link = link_class(prefix=prefix, hash=node.hash, size=node.size)
             node = node_class(
-                value    = parent.value,
-                children = (list(x for x in parent.children[:idx]) +
-                            list((link_class(prefix=prefix, node=node),)) +
-                            list(x for x in parent.children[idx+1:])))
+                value       = parent.value,
+                children    = (list(x for x in parent.children[:idx]) + list((link,)) +
+                               list(x for x in parent.children[idx+1:])),
+                prune_value = parent.prune_value)
         for attr in self.__slots__:
             setattr(self, attr, getattr(node, attr, None))
 
@@ -518,10 +537,11 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
                         node   = old_node.children[idx].node,
                         hash   = old_node.children[idx]._hash),))
                 new_node = node_class(
-                    value    = old_node.value,
-                    children = (list(x for x in old_node.children[:idx]) +
-                                list((link_class(prefix=common_prefix, node=inner_node),)) +
-                                list(x for x in old_node.children[idx+1:])))
+                    value       = old_node.value,
+                    children    = (list(x for x in old_node.children[:idx]) +
+                                   list((link_class(prefix=common_prefix, node=inner_node),)) +
+                                   list(x for x in old_node.children[idx+1:])),
+                    prune_value = old_node.prune_value)
 
             elif len(common_prefix):
                 leaf_node = node_class(
@@ -535,20 +555,22 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
                         old_node.children[idx].prefix[len(common_prefix):]:
                             old_node.children[idx].node})
                 new_node = node_class(
-                    value    = old_node.value,
-                    children = (list(x for x in old_node.children[:idx]) +
-                                list((link_class(prefix=common_prefix, node=inner_node),)) +
-                                list(x for x in old_node.children[idx+1:])))
+                    value       = old_node.value,
+                    children    = (list(x for x in old_node.children[:idx]) +
+                                   list((link_class(prefix=common_prefix, node=inner_node),)) +
+                                   list(x for x in old_node.children[idx+1:])),
+                    prune_value = old_node.prune_value)
 
             else:
                 leaf_node = node_class(
                     value    = value,
                     children = {})
                 new_node = node_class(
-                    value    = old_node.value,
-                    children = (list(x for x in old_node.children[:idx]) +
-                                list((link_class(prefix=remaining_key, node=leaf_node),)) +
-                                list(x for x in old_node.children[idx:])))
+                    value       = old_node.value,
+                    children    = (list(x for x in old_node.children[:idx]) +
+                                   list((link_class(prefix=remaining_key, node=leaf_node),)) +
+                                   list(x for x in old_node.children[idx:])),
+                    prune_value = old_node.prune_value)
 
         self._propogate(new_node, path=path)
 
@@ -579,6 +601,90 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
         if key not in self:
             self.update(((key, value),))
 
+    def _trim(self, key):
+        key, key_ = self._prepare_key(key), key
+
+        link_class = getattr(self, 'get_link_class',
+            lambda: getattr(self, 'link_class', PatriciaLink))()
+        node_class = getattr(self, 'get_node_class',
+            lambda: getattr(self, 'node_class', self.__class__))()
+
+        path = list()
+        prefix, old_node = self._get_node_by_key(key, path=path)
+
+        if key == prefix:
+            new_node = node_class(
+                value       = old_node.value,
+                children    = (link_class(prefix=link.prefix, hash=link.hash, size=link.size)
+                               for link in old_node.children),
+                prune_value = old_node.value is not None and True or None)
+
+            length = old_node.length
+
+        else:
+            remaining_key = key[len(prefix):]
+            idx = bisect_left(old_node.children, link_class(
+                prefix=remaining_key[:1], hash=0, size=0))
+            if idx not in xrange(len(old_node.children)):
+                return 0
+
+            link = old_node.children[idx]
+            if link.pruned:
+                return 0
+
+            common_prefix = commonprefix([link.prefix, remaining_key])
+            if common_prefix != remaining_key:
+                return 0
+
+            new_node = node_class(
+                value       = old_node.value,
+                children    = (list(x for x in old_node.children[:idx]) +
+                               list((link_class(prefix = link.prefix,
+                                                hash   = link.hash,
+                                                size   = link.size),)) +
+                               list(x for x in old_node.children[idx+1:])),
+                prune_value = old_node.prune_value)
+
+            length = link.length
+
+        self._propogate(new_node, path=path)
+        return length
+
+    def trim(self, prefixes):
+        "Prunes any keys beginning with the specified the specified prefixes."
+        _prefixes, prefixes = set(map(lambda k:self._prepare_key(k), prefixes)), list()
+        for t in lookahead(sorted(_prefixes)):
+            if t[1] is not None:
+                if t[0] == commonprefix(t):
+                    continue
+            prefixes.append(t[0])
+        length = 0
+        for prefix in prefixes:
+            length += self._trim(prefix)
+        return length
+
+    def _prune(self, key):
+        link_class = getattr(self, 'get_link_class',
+            lambda: getattr(self, 'link_class', PatriciaLink))()
+        node_class = getattr(self, 'get_node_class',
+            lambda: getattr(self, 'node_class', self.__class__))()
+
+        key, _key, path = self._prepare_key(key), key, list()
+        prefix, old_node = self._get_node_by_key(key, path=path)
+        if key != prefix or old_node.value is None or old_node.prune_value:
+            raise KeyError(_key)
+
+        new_node = node_class(
+            value       = old_node.value,
+            children    = old_node.children,
+            prune_value = True)
+        self._propogate(new_node, path=path)
+
+    def prune(self, keys):
+        "Removes the specified keys without changing any hash values."
+        for key in keys:
+            self._prune(key)
+
     def _delete(self, key):
         key, key_ = self._prepare_key(key), key
 
@@ -599,21 +705,23 @@ class BasePatriciaDict(SerializableMixin, HashableMixin):
             # Remove from parent
             parent, idx, prefix = path.pop()
             new_node = node_class(
-                value    = parent.value,
-                children = (list(x for x in parent.children[:idx]) +
-                            list(x for x in parent.children[idx+1:])))
+                value       = parent.value,
+                children    = (list(x for x in parent.children[:idx]) +
+                               list(x for x in parent.children[idx+1:])),
+                prune_value = parent.prune_value)
 
         while all([path, len(new_node.children)==1, new_node.value is None]):
             # Squash with child node
             parent, idx, prefix = path.pop()
             new_node = node_class(
-                value    = parent.value,
-                children = (list(x for x in parent.children[:idx]) +
-                            list((link_class(
-                                prefix = prefix + new_node.children[0].prefix,
-                                node   = new_node.children[0].node,
-                                hash   = new_node.children[0]._hash),)) +
-                            list(x for x in parent.children[idx+1:])))
+                value       = parent.value,
+                children    = (list(x for x in parent.children[:idx]) +
+                               list((link_class(
+                                   prefix = prefix + new_node.children[0].prefix,
+                                   node   = new_node.children[0].node,
+                                   hash   = new_node.children[0]._hash),)) +
+                               list(x for x in parent.children[idx+1:])),
+                prune_value = parent.prune_value)
 
         self._propogate(new_node, path=path)
 
